@@ -28,20 +28,6 @@ from .cpp.delaunay_class3 import DelaunayTriangulation3 as DT3
 
 __all__ = ["sliver_removal", "generate_mesh"]
 
-opts = {
-    "verbose": 1,
-    "max_iter": 50,
-    "seed": 0,
-    "perform_checks": False,
-    "pfix": None,
-    "axis": 1,
-    "min_dh_angle_bound": 10.0,
-    "max_dh_angle_bound": 180.0,
-    "points": None,
-    "delta_t": 0.30,
-    "geps_mult": 0.1,
-}
-
 
 def silence(func):
     def wrapper(*args, **kwargs):
@@ -117,10 +103,22 @@ def sliver_removal(points, domain, edge_length, comm=None, **kwargs):  # noqa: C
             warnings.warn("Sliver removal only works in serial for now")
         return True, True
 
-    opts.update(kwargs)
+    sliver_opts = {
+        "verbose": 1,
+        "max_iter": 50,
+        "perform_checks": False,
+        "axis": 1,
+        "min_dh_angle_bound": 10.0,
+        "max_dh_angle_bound": 180.0,
+        "points": None,
+        "delta_t": 0.30,
+        "geps_mult": 0.1,
+    }
+
+    sliver_opts.update(kwargs)
     _parse_kwargs(kwargs)
 
-    verbosity1, verbosity2 = _select_verbosity(opts)
+    verbosity1, verbosity2 = _select_verbosity(sliver_opts)
 
     @verbosity1
     def print_msg1(msg):
@@ -135,7 +133,7 @@ def sliver_removal(points, domain, edge_length, comm=None, **kwargs):  # noqa: C
         if comm.rank == 0:
             raise Exception("Mesh improvement currently on works in 3D")
 
-    fd, bbox0 = _unpack_domain(domain)
+    fd, bbox0, _ = _unpack_domain(domain, sliver_opts)
 
     fh, bbox1, hmin = _unpack_sizing(edge_length)
 
@@ -156,25 +154,23 @@ def sliver_removal(points, domain, edge_length, comm=None, **kwargs):  # noqa: C
     if hmin is not None:
         h0 = hmin
     else:
-        h0 = opts["h0"]
+        h0 = sliver_opts["h0"]
 
     # check h0
     if h0 < 0:
         raise ValueError("`h0` must be > 0")
 
-    deps = np.sqrt(np.finfo(np.double).eps) * h0
-
-    if opts["max_iter"] < 0:
+    if sliver_opts["max_iter"] < 0:
         raise ValueError("`max_iter` must be > 0")
-    max_iter = opts["max_iter"]
+    max_iter = sliver_opts["max_iter"]
 
     print_msg1(
         "Will attempt " + str(max_iter) + " iterations to bound the dihedral angles..."
     )
 
-    geps = opts["geps_mult"] * h0
-    min_dh_bound = opts["min_dh_angle_bound"] * math.pi / 180
-    max_dh_bound = opts["max_dh_angle_bound"] * math.pi / 180
+    geps = sliver_opts["geps_mult"] * h0
+    min_dh_bound = sliver_opts["min_dh_angle_bound"] * math.pi / 180
+    max_dh_bound = sliver_opts["max_dh_angle_bound"] * math.pi / 180
 
     print_msg1(
         "Enforcing a min. dihedral bound of: "
@@ -228,7 +224,7 @@ def sliver_removal(points, domain, edge_length, comm=None, **kwargs):  # noqa: C
             print_msg1(
                 "FAILURE: Termination...maximum number of iterations reached.",
             )
-            p, t = _termination(p, t, opts, comm, sliver=True)
+            p, t = _termination(p, t, sliver_opts, comm, sliver=True)
             break
 
         print_msg1(
@@ -270,9 +266,6 @@ def sliver_removal(points, domain, edge_length, comm=None, **kwargs):  # noqa: C
         # perturb push % of minimum mesh size
         p[move] += push * h0 * perturb
 
-        # bring outside points back to the boundary
-        p = _project_points_back_newton(p, fd, deps)
-
         count += 1
 
         end = time.time()
@@ -329,13 +322,23 @@ def generate_mesh(domain, edge_length, comm=None, **kwargs):  # noqa: C901
 
     """
     comm = comm or MPI.COMM_WORLD
-
+    gen_opts = {
+        "verbose": 1,
+        "max_iter": 50,
+        "seed": 0,
+        "perform_checks": False,
+        "pfix": None,
+        "axis": 1,
+        "points": None,
+        "delta_t": 0.30,
+        "geps_mult": 0.1,
+    }
     # check call was correct
-    opts.update(kwargs)
+    gen_opts.update(kwargs)
     _parse_kwargs(kwargs)
 
     # verbosity decorators
-    verbosity1, verbosity2 = _select_verbosity(opts)
+    verbosity1, verbosity2 = _select_verbosity(gen_opts)
 
     @verbosity1
     def print_msg1(msg):
@@ -346,7 +349,11 @@ def generate_mesh(domain, edge_length, comm=None, **kwargs):  # noqa: C901
         print(msg, flush=True)
 
     # unpack domain
-    fd, bbox0 = _unpack_domain(domain)
+    fd, bbox0, corners = _unpack_domain(domain, gen_opts)
+
+    # discard corners for now
+    if not np.isscalar(edge_length):
+        corners = None
 
     fh, bbox1, hmin = _unpack_sizing(edge_length)
 
@@ -376,27 +383,38 @@ def generate_mesh(domain, edge_length, comm=None, **kwargs):  # noqa: C901
     if hmin is not None:
         h0 = hmin
     else:
-        h0 = opts["h0"]
+        h0 = gen_opts["h0"]
 
     # check h0
     if h0 < 0:
         raise ValueError("`h0` must be > 0")
 
-    if opts["max_iter"] < 0:
-        raise ValueError("`max_iter` must be > 0")
-    max_iter = opts["max_iter"]
-
     # these parameters originate from the original DistMesh
     L0mult = 1 + 0.4 / 2 ** (dim - 1)
-    delta_t = opts["delta_t"]
-    geps = opts["geps_mult"] * h0
+    delta_t = gen_opts["delta_t"]
+    geps = gen_opts["geps_mult"] * h0
     deps = np.sqrt(np.finfo(np.double).eps) * h0
 
     DT = _select_cgal_dim(dim)
 
-    pfix, nfix = _unpack_pfix(dim, opts, comm)
+    pfix, nfix = _unpack_pfix(dim, gen_opts, comm)
+    if corners is not None and comm.size == 1:
+        # keep corners only near level set
+        corners = corners[fd(corners) > -1000 * deps]
+        pfix = np.append(pfix, corners, axis=0)
+        nfix = len(pfix)
 
-    fh, p, extents = _initialize_points(dim, geps, bbox, fh, fd, h0, opts, pfix, comm)
+    if comm.rank == 0:
+        print_msg1("Constraining " + str(nfix) + " fixed points..")
+
+    fh, p, extents = _initialize_points(
+        dim, geps, bbox, fh, fd, h0, gen_opts, pfix, comm
+    )
+
+    if gen_opts["max_iter"] < 0:
+        raise ValueError("`max_iter` must be > 0")
+
+    max_iter = gen_opts["max_iter"]
 
     N = p.shape[0]
 
@@ -438,7 +456,7 @@ def generate_mesh(domain, edge_length, comm=None, **kwargs):  # noqa: C901
                 print_msg1(
                     "Termination reached...maximum number of iterations reached.",
                 )
-            p, t = _termination(p, t, opts, comm)
+            p, t = _termination(p, t, gen_opts, comm)
             if comm.rank == 0:
                 p = _improve_level_set_newton(p, t, fd, deps, deps * 1000)
             break
@@ -465,6 +483,9 @@ def generate_mesh(domain, edge_length, comm=None, **kwargs):  # noqa: C901
                 "Iteration #%d, max movement is %f, there are %d vertices and %d cells"
                 % (count + 1, maxdp, len(p), len(t)),
             )
+            assert (
+                maxdp < 50 * h0
+            ), "max movement indicates there's a convergence problem"
 
         count += 1
 
@@ -532,16 +553,25 @@ def _unpack_sizing(edge_length):
     return fh, bbox, hmin
 
 
-def _unpack_domain(domain):
-    if isinstance(domain, geometry.Rectangle):
-        bbox = (domain.x1, domain.x2, domain.y1, domain.y2)
-        fd = domain.eval
-    elif isinstance(domain, geometry.Cube):
-        bbox = (domain.x1, domain.x2, domain.y1, domain.y2, domain.z1, domain.z2)
-        fd = domain.eval
-    elif isinstance(domain, geometry.Disk):
-        bbox = (domain.x1, domain.x2, domain.y1, domain.y2)
+def _unpack_domain(domain, opts):
+    corners = None
+    domains = [
+        geometry.Ball,
+        geometry.Rectangle,
+        geometry.Disk,
+        geometry.Cube,
+        geometry.Cylinder,
+        geometry.Cube,
+        geometry.Torus,
+        geometry.Prism,
+        geometry.Union,
+        geometry.Intersection,
+        geometry.Difference,
+    ]
+    if np.any([isinstance(domain, d) for d in domains]):
+        bbox = domain.bbox
         fd = domain.eval
+        corners = domain.corners
     elif callable(domain):
         # get the bbox from the name value pairs or quit
         bbox = opts["bbox"]
@@ -549,7 +579,7 @@ def _unpack_domain(domain):
     else:
         raise ValueError("`domain` must be a function or a :class:`geometry` object")
     _check_bbox(bbox)
-    return fd, bbox
+    return fd, bbox, corners
 
 
 def _parse_kwargs(kwargs):
@@ -603,7 +633,6 @@ def _get_edges(t):
     return geometry.unique_edges(edges)
 
 
-# @profile
 def _compute_forces(p, t, fh, h0, L0mult):
     """Compute the forces on each edge based on the sizing function"""
     dim = p.shape[1]
@@ -657,11 +686,9 @@ def _remove_triangles_outside(p, t, fd, geps):
 def _improve_level_set_newton(p, t, fd, deps, tol):
     """Reduce level set error by using Newton's minimization method"""
     dim = p.shape[1]
-    max, abs = np.amax, np.abs
     bid = geometry.get_boundary_vertices(t, dim)
-    it = 0
     d = fd(p[bid])
-    while max(abs(d)) > tol and it < 10:
+    for _ in range(5):
 
         def _deps_vec(i):
             a = [0] * dim
@@ -672,7 +699,6 @@ def _improve_level_set_newton(p, t, fd, deps, tol):
         dgrad2 = sum(dgrad ** 2 for dgrad in dgrads)
         dgrad2 = np.where(dgrad2 < deps, deps, dgrad2)
         p[bid] -= (d * np.vstack(dgrads) / dgrad2).T  # Project
-        it += 1
     return p
 
 
@@ -744,7 +770,7 @@ def _generate_initial_points(h0, geps, dim, bbox, fh, fd, pfix, comm, opts):
             p[np.random.rand(p.shape[0]) < r0m ** dim / r0 ** dim],
         )
     )
-    extents = _form_extents(p, h0, comm)
+    extents = _form_extents(p, h0, comm, opts)
     return fh, p, extents
 
 
@@ -752,7 +778,6 @@ def _initialize_points(dim, geps, bbox, fh, fd, h0, opts, pfix, comm):
     """Form initial point set to mesh with"""
     points = opts["points"]
     if points is None:
-        # def _generate_initial_points(h0, geps, dim, bbox, fh, fd, pfix, comm, opts):
         fh, p, extents = _generate_initial_points(
             h0, geps, dim, bbox, fh, fd, pfix, comm, opts
         )
@@ -761,7 +786,7 @@ def _initialize_points(dim, geps, bbox, fh, fd, h0, opts, pfix, comm):
     return fh, p, extents
 
 
-def _form_extents(p, h0, comm):
+def _form_extents(p, h0, comm, opts):
     dim = p.shape[1]
     _axis = opts["axis"]
     if comm.size > 1:
@@ -781,17 +806,9 @@ def _dist(p1, p2):
 
 def _unpack_pfix(dim, opts, comm):
     """Unpack fixed points"""
-    if opts["pfix"] is not None:
-        if comm.size > 1:
-            raise Exception("Fixed points are not yet supported in parallel.")
-        else:
-            pfix = np.array(opts["pfix"], dtype="d")
-            nfix = len(pfix)
-        if comm.rank == 0:
-            print(
-                "Constraining " + str(nfix) + " fixed points..",
-                flush=True,
-            )
+    if opts["pfix"] is not None and comm.size == 1:
+        pfix = np.array(opts["pfix"], dtype="d")
+        nfix = len(pfix)
     else:
         pfix = np.empty((0, dim))
         nfix = 0
