@@ -1,42 +1,40 @@
+#include <algorithm>
 #include <pybind11/complex.h>
 #include <pybind11/numpy.h>
 #include <pybind11/stl.h>
-#include <vector>
 #include <set>
-#include <algorithm>
 #include <tuple>
+#include <vector>
 
-
-#include <iostream>
 #include <ctime>
+#include <iostream>
 
 namespace py = pybind11;
 
-class Timer
-{
+class Timer {
 public:
-    Timer() { clock_gettime(CLOCK_REALTIME, &beg_); }
+  Timer() { clock_gettime(CLOCK_REALTIME, &beg_); }
 
-    double elapsed() {
-        clock_gettime(CLOCK_REALTIME, &end_);
-        return end_.tv_sec - beg_.tv_sec +
-            (end_.tv_nsec - beg_.tv_nsec) / 1000000000.;
-    }
+  double elapsed() {
+    clock_gettime(CLOCK_REALTIME, &end_);
+    return end_.tv_sec - beg_.tv_sec +
+           (end_.tv_nsec - beg_.tv_nsec) / 1000000000.;
+  }
 
-    void reset() { clock_gettime(CLOCK_REALTIME, &beg_); }
+  void reset() { clock_gettime(CLOCK_REALTIME, &beg_); }
 
 private:
-    timespec beg_, end_;
+  timespec beg_, end_;
 };
 
 template <typename T>
 std::vector<T> vectorSortIntArr(std::vector<std::array<T, 2>> v) {
   std::sort(v.begin(), v.end());
-  //double t = tmr.elapsed();
-  //tmr.reset();
+  // double t = tmr.elapsed();
+  // tmr.reset();
   auto iter = std::unique(v.begin(), v.end());
-  //t = tmr.elapsed();
-  //std::cout << t << std::endl;
+  // t = tmr.elapsed();
+  // std::cout << t << std::endl;
 
   size_t len = iter - v.begin();
   std::vector<T> outvec;
@@ -49,7 +47,7 @@ std::vector<T> vectorSortIntArr(std::vector<std::array<T, 2>> v) {
 }
 
 py::array unique_edges(
-    py::array_t<int, py::array::c_style | py::array::forcecast> edges){
+    py::array_t<int, py::array::c_style | py::array::forcecast> edges) {
 
   std::vector<int> cedges(edges.size());
   std::memcpy(cedges.data(), edges.data(), edges.size() * sizeof(int));
@@ -57,26 +55,26 @@ py::array unique_edges(
   std::vector<std::array<int, 2>> tl;
 
   tl.reserve(cedges.size());
-  for(size_t i=0;i<cedges.size();i+=2){
-     tl.push_back({std::min(cedges[i],cedges[i+1]),std::max(cedges[i],cedges[i+1])});
+  for (size_t i = 0; i < cedges.size(); i += 2) {
+    tl.push_back({std::min(cedges[i], cedges[i + 1]),
+                  std::max(cedges[i], cedges[i + 1])});
   }
 
   auto u_edges = vectorSortIntArr<int>(std::move(tl));
 
   int num_edges = u_edges.size();
   ssize_t sint = sizeof(int);
-  std::vector<ssize_t> shape = {num_edges/2, 2};
+  std::vector<ssize_t> shape = {num_edges / 2, 2};
   std::vector<ssize_t> strides = {sint * 2, sint};
   return py::array(
       py::buffer_info(u_edges.data(), /* data as contiguous array  */
-                      sizeof(int),           /* size of one scalar        */
+                      sizeof(int),    /* size of one scalar        */
                       py::format_descriptor<int>::format(), /* data type */
                       2,      /* number of dimensions      */
                       shape,  /* shape of the matrix       */
                       strides /* strides for each axis     */
                       ));
-  }
-
+}
 
 double l2_norm(const std::array<double, 3> &u) {
   double accum = 0.;
